@@ -1,5 +1,5 @@
 /*
- *  $Id: sctpassociation.cc,v 1.11 2004/07/30 11:03:43 dreibh Exp $
+ *  $Id: sctpassociation.cc 681 2005-08-03 11:23:45Z dreibh $
  *
  * SocketAPI implementation for the sctplib.
  * Copyright (C) 1999-2003 by Thomas Dreibholz
@@ -73,6 +73,7 @@ SCTPAssociation::SCTPAssociation(SCTPSocket*        socket,
    FirstPreEstablishmentPacket   = NULL;
    LastPreEstablishmentPacket    = NULL;
    PreEstablishmentAddressList   = NULL;
+   PeeledOff                     = false;
 
    EstablishCondition.setName("SCTPAssociation::EstablishCondition");
    ShutdownCompleteCondition.setName("SCTPAssociation::ShutdownCompleteCondition");
@@ -162,7 +163,7 @@ SCTPAssociation::~SCTPAssociation()
    PreEstablishmentPacket* packet = FirstPreEstablishmentPacket;
    while(packet != NULL) {
       PreEstablishmentPacket* nextPacket = packet->Next;
-      delete packet->Data;
+      delete [] packet->Data;
       packet->Data = NULL;
       delete packet;
       packet = nextPacket;
@@ -682,6 +683,13 @@ int SCTPAssociation::getTrafficClass(const int streamID)
 bool SCTPAssociation::setTrafficClass(const card8 trafficClass,
                                       const int   streamID)
 {
+   return(true);
+
+/*
+   To set a traffic class, some changes would be necessary for the sctplib:
+   An association may be in a state where it is not fully established yet.
+   This code here will not work in this case!
+
    bool result = false;
    SCTPSocketMaster::MasterInstance.lock();
    SCTP_Association_Status status;
@@ -693,6 +701,7 @@ bool SCTPAssociation::setTrafficClass(const card8 trafficClass,
    }
    SCTPSocketMaster::MasterInstance.unlock();
    return(result);
+*/
 }
 
 
