@@ -1,5 +1,5 @@
 /*
- *  $Id: sctpsocket.cc,v 1.28 2005/03/04 13:22:14 dreibh Exp $
+ *  $Id: sctpsocket.cc 689 2005-08-04 15:49:54Z dreibh $
  *
  * SocketAPI implementation for the sctplib.
  * Copyright (C) 1999-2003 by Thomas Dreibholz
@@ -41,34 +41,31 @@
 #include "sctpsocketmaster.h"
 
 
-/*
-#define PRINT_BIND
-#define PRINT_UNBIND
-#define PRINT_ADDIP
-#define PRINT_ACCEPT
-#define PRINT_ASSOCIATE
-#define PRINT_NEW_ASSOCIATIONS
-#define PRINT_SEND_TO_ALL
-#define PRINT_SHUTDOWNS
-#define PRINT_PRSCTP
-#define PRINT_NOTIFICATION_SKIP
-#define PRINT_DATA
-#define PRINT_RECVSTATUS
-#define PRINT_SENDSTATUS
-#define PRINT_SETPRIMARY
-*/
-
-/*
-#define PRINT_AUTOCLOSE_TIMEOUT
-#define PRINT_AUTOCLOSE_CHECK
-
-#define PRINT_RECVWAIT
-#define PRINT_ISSHUTDOWN
-#define PRINT_PATHFORINDEX
-#define PRINT_ASSOCSEARCH
-#define PRINT_ASSOC_USECOUNT
-#define PRINT_RTO
-*/
+// #define PRINT_BIND
+// #define PRINT_UNBIND
+// #define PRINT_ADDIP
+// #define PRINT_ACCEPT
+// #define PRINT_ASSOCIATE
+// #define PRINT_NEW_ASSOCIATIONS
+// #define PRINT_SEND_TO_ALL
+// #define PRINT_SHUTDOWNS
+// #define PRINT_PRSCTP
+// #define PRINT_NOTIFICATION_SKIP
+// #define PRINT_DATA
+// #define PRINT_RECVSTATUS
+// #define PRINT_SENDSTATUS
+// #define PRINT_SETPRIMARY
+//
+//
+// #define PRINT_AUTOCLOSE_TIMEOUT
+// #define PRINT_AUTOCLOSE_CHECK
+//
+// #define PRINT_RECVWAIT
+// #define PRINT_ISSHUTDOWN
+// #define PRINT_PATHFORINDEX
+// #define PRINT_ASSOCSEARCH
+// #define PRINT_ASSOC_USECOUNT
+// #define PRINT_RTO
 
 
 // ###### Constructor #######################################################
@@ -86,6 +83,8 @@ SCTPSocket::SCTPSocket(const int family, const cardinal flags)
    HasException        = false;
    Family              = family;
 
+   AutoCloseRecursion  = false;
+
    EstablishCondition.setName("SCTPSocket::EstablishCondition");
    ReadUpdateCondition.setName("SCTPSocket::ReadUpdateCondition");
    WriteUpdateCondition.setName("SCTPSocket::WriteUpdateCondition");
@@ -871,17 +870,22 @@ int SCTPSocket::internalReceive(SCTPNotificationQueue& queue,
             flags |= MSG_NOTIFICATION;
          }
          else {
+            if(flags & MSG_PEEK) {
+               notification.ContentPosition = 0;
+               queue.updateNotification(notification);
+               updatedNotification = true;
+            }
             flags |= (MSG_EOR|MSG_NOTIFICATION);
          }
 
 #ifdef PRINT_DATA
-            cout << "Received " << bufferSize << " bytes notification data from association " << assocID << ", stream " << streamID << ":" << endl;
-            for(size_t i = 0;i < bufferSize;i++) {
-               char str[32];
-               snprintf((char*)&str,sizeof(str),"%02x ",((unsigned char*)buffer)[i]);
-               cout << str;
-            }
-            cout << endl;
+         cout << "Received " << bufferSize << " bytes notification data from association " << assocID << ", stream " << streamID << ":" << endl;
+         for(size_t i = 0;i < bufferSize;i++) {
+            char str[32];
+            snprintf((char*)&str,sizeof(str),"%02x ",((unsigned char*)buffer)[i]);
+            cout << str;
+         }
+         cout << endl;
 #endif
          result = (int)bufferSize;
       }
@@ -1043,7 +1047,10 @@ int SCTPSocket::internalSend(const char*          buffer,
    if(result == SCTP_PARAMETER_PROBLEM) {
       return(-EINVAL);
    }
-   return(EIO);
+   if(result == SCTP_QUEUE_EXCEEDED) {
+      return(-ENOBUFS);
+   }
+   return(-EIO);
 }
 
 
@@ -1408,8 +1415,9 @@ SCTPAssociation* SCTPSocket::peelOff(const SocketAddress& destinationAddress)
          if( (!iterator->second->IsShuttingDown)               &&
              (destinationAddress.getPort() == status.destPort) &&
              (destinationAddress.getAddressString(InternetAddress::PF_HidePort|InternetAddress::PF_Address|InternetAddress::PF_Legacy) == String((const char*)&status.primaryDestinationAddress)) ) {
-            ConnectionlessAssociationList.erase(iterator);
             association = iterator->second;
+            association->PeeledOff = true;
+            ConnectionlessAssociationList.erase(iterator);
             break;
          }
       }
@@ -1432,6 +1440,7 @@ SCTPAssociation* SCTPSocket::peelOff(const unsigned int assocID)
    if( (iterator != ConnectionlessAssociationList.end()) &&
        (!iterator->second->IsShuttingDown) ) {
       association = iterator->second;
+      association->PeeledOff = true;
       ConnectionlessAssociationList.erase(iterator);
    }
    SCTPSocketMaster::MasterInstance.unlock();
@@ -1853,53 +1862,68 @@ bool SCTPSocket::deleteAddress(const unsigned int   assocID,
 // ###### Check for necessity to auto-close associations ####################
 void SCTPSocket::checkAutoClose()
 {
-   const card64 now = getMicroTime();
-   multimap<unsigned int, SCTPAssociation*>::iterator iterator =
-      ConnectionlessAssociationList.begin();
-   while(iterator != ConnectionlessAssociationList.end()) {
-
-      SCTPAssociation* association = iterator->second;
-#ifdef PRINT_AUTOCLOSE_CHECK
-      cout << "AutoConnect: Check for AutoClose:" << endl
-           << "   AssocID          = " << association->getID() << endl
-           << "   UseCount         = " << association->UseCount << endl
-           << "   LastUsage        = " << now - association->LastUsage << endl
-           << "   AutoCloseTimeout = " << AutoCloseTimeout << endl;
-#endif
-      if((association->UseCount == 0) &&
-         (AutoCloseTimeout > 0) &&
-         (now - association->LastUsage > AutoCloseTimeout)) {
-#ifdef PRINT_AUTOCLOSE_TIMEOUT
-         const unsigned int assocID = association->getID();
-         cout << "AutoConnect: Doing shutdown of association #" << assocID << " due to timeout" << endl;
-#endif
-         association->shutdown();
-         iterator++;
-      }
-      else if((association->ShutdownCompleteNotification)         ||
-              (association->CommunicationLostNotification)) {
-#ifdef PRINT_AUTOCLOSE_TIMEOUT
-         const unsigned int assocID = association->getID();
-         cout << "AutoConnect: Removing association #" << assocID << ": ";
-         if(association->ShutdownCompleteNotification) {
-            cout << "shutdown complete";
-         }
-         else if(association->CommunicationLostNotification) {
-            cout << "communication lost";
-         }
-         cout << "..." << endl;
-#endif
-         delete association;
-#ifdef PRINT_AUTOCLOSE_TIMEOUT
-         cout << "AutoConnect: AutoClose of association #" << assocID << " completed!" << endl;
-#endif
-         ConnectionlessAssociationList.erase(iterator);
-         iterator = ConnectionlessAssociationList.begin();
-      }
-      else {
-         iterator++;
-      }
+   if(AutoCloseRecursion) {
+      AutoCloseNewCheckRequired = true;
+      return;
    }
+   AutoCloseRecursion = true;
+
+   do {
+      AutoCloseNewCheckRequired = false;
+
+      const card64 now = getMicroTime();
+      multimap<unsigned int, SCTPAssociation*>::iterator iterator =
+         ConnectionlessAssociationList.begin();
+      while(iterator != ConnectionlessAssociationList.end()) {
+         SCTPAssociation* association = iterator->second;
+#ifdef PRINT_AUTOCLOSE_CHECK
+         cout << "AutoConnect: Check for AutoClose:" << endl
+            << "   AssocID          = " << association->getID() << endl
+            << "   UseCount         = " << association->UseCount << endl
+            << "   LastUsage        = " << now - association->LastUsage << endl
+            << "   AutoCloseTimeout = " << AutoCloseTimeout << endl;
+#endif
+
+         if((association->UseCount == 0) &&
+            (AutoCloseTimeout > 0) &&
+            (now - association->LastUsage > AutoCloseTimeout)) {
+#ifdef PRINT_AUTOCLOSE_TIMEOUT
+            const unsigned int assocID = association->getID();
+            cout << "AutoConnect: Doing shutdown of association #" << assocID << " due to timeout" << endl;
+#endif
+            iterator++;   // Important! shutdown() may invalidate iterator!
+            association->shutdown();
+         }
+         else if((association->ShutdownCompleteNotification)         ||
+               (association->CommunicationLostNotification)) {
+#ifdef PRINT_AUTOCLOSE_TIMEOUT
+            const unsigned int assocID = association->getID();
+            cout << "AutoConnect: Removing association #" << assocID << ": ";
+            if(association->ShutdownCompleteNotification) {
+               cout << "shutdown complete";
+            }
+            else if(association->CommunicationLostNotification) {
+               cout << "communication lost";
+            }
+            cout << "..." << endl;
+#endif
+
+            // Important! Removal will invalidate iterator!
+            multimap<unsigned int, SCTPAssociation*>::iterator delIterator = iterator;
+            iterator++;
+            ConnectionlessAssociationList.erase(delIterator);
+
+            delete association;
+#ifdef PRINT_AUTOCLOSE_TIMEOUT
+            cout << "AutoConnect: AutoClose of association #" << assocID << " completed!" << endl;
+#endif
+         }
+         else {
+            iterator++;
+         }
+      }
+   } while(AutoCloseNewCheckRequired == true);
+   AutoCloseRecursion = false;
 }
 
 
