@@ -1,5 +1,5 @@
 /*
- *  $Id: sctpsocket.h,v 1.8 2004/11/23 10:13:45 dreibh Exp $
+ *  $Id: sctpsocket.h 664 2005-07-28 12:18:32Z dreibh $
  *
  * SocketAPI implementation for the sctplib.
  * Copyright (C) 1999-2003 by Thomas Dreibholz
@@ -590,6 +590,9 @@ class SCTPSocket
    bool                                     ReadReady;
    bool                                     HasException;
 
+   bool                                     AutoCloseRecursion;
+   bool                                     AutoCloseNewCheckRequired;
+
 #if (SCTPLIB_VERSION == SCTPLIB_1_0_0_PRE19)
    unsigned int                             NoOfLocalAddresses;
    unsigned char                            LocalAddressList[SCTP_MAX_NUM_ADDRESSES][SCTP_MAX_IP_LEN];
