@@ -1,5 +1,5 @@
 /*
- *  $Id: sctpsocketmaster.cc,v 1.18 2004/11/23 10:13:45 dreibh Exp $
+ *  $Id: sctpsocketmaster.cc 660 2005-07-22 14:30:13Z dreibh $
  *
  * SocketAPI implementation for the sctplib.
  * Copyright (C) 1999-2003 by Thomas Dreibholz
@@ -51,7 +51,7 @@
 
 // #define PRINT_NOTIFICATIONS
 // #define PRINT_ARRIVENOTIFICATION
-
+//
 // #define PRINT_PIPE
 // #define PRINT_GC
 // #define PRINT_ASSOC_USECOUNT
@@ -573,20 +573,24 @@ void SCTPSocketMaster::networkStatusChangeNotif(unsigned int assocID,
 #else
 #error Wrong sctplib version!
 #endif
+   if(ok != 0) {
+#ifndef DISABLE_WARNINGS
+      cerr << "INTERNAL ERROR: SCTPSocketMaster::networkStatusChangeNotif() - sctp_getPathStatus() failed!" << endl;
+#endif
+      return;
+   }
 
 
    // ====== Get new destination address ====================================
    SocketAddress* destination = NULL;
-   if(ok == 0) {
-      destination = SocketAddress::createSocketAddress(
+   destination = SocketAddress::createSocketAddress(
                                       SocketAddress::PF_HidePort,
                                       (char*)&pathStatus.destinationAddress);
-      if(destination == NULL) {
+   if(destination == NULL) {
 #ifndef DISABLE_WARNINGS
-         cerr << "INTERNAL ERROR: SCTPSocketMaster::networkStatusChangeNotif() - Bad destination address!" << endl;
+      cerr << "INTERNAL ERROR: SCTPSocketMaster::networkStatusChangeNotif() - Bad destination address!" << endl;
 #endif
-         return;
-      }
+      return;
    }
 
    // ====== Generate "Network Status Change" notification ==================
@@ -853,8 +857,8 @@ void SCTPSocketMaster::communicationLostNotif(unsigned int assocID,
          association->EstablishCondition.broadcast();
          association->ReadyForTransmit.broadcast();
       }
+      socket->checkAutoClose();
    }
-   socket->checkAutoClose();
 }
 
 
@@ -1201,7 +1205,7 @@ void SCTPSocketMaster::addNotification(SCTPSocket*             socket,
                                        const SCTPNotification& notification)
 {
    // ====== Get notification flags =========================================
-   SCTPAssociation* association = socket->getAssociationForAssociationID(assocID,false);
+   SCTPAssociation* association = socket->getAssociationForAssociationID(assocID, false);
    if(association == NULL) {
       // Association not found -> already closed.
       return;
@@ -1210,10 +1214,6 @@ void SCTPSocketMaster::addNotification(SCTPSocket*             socket,
 
    // ====== Check, if notification has to be added =========================
    if((notification.Content.sn_header.sn_type == SCTP_DATA_ARRIVE)    ||
-/* ????? Deprecated!
-      ((notification.Content.sn_header.sn_type == SCTP_ASSOC_CHANGE) && (notification.Content.sn_assoc_change.sac_state == SCTP_SHUTDOWN_COMP)) ||
-      ((notification.Content.sn_header.sn_type == SCTP_ASSOC_CHANGE) && (notification.Content.sn_assoc_change.sac_state == SCTP_COMM_LOST))     ||
-*/
       ((notification.Content.sn_header.sn_type == SCTP_ASSOC_CHANGE)     && (notificationFlags & SCTP_RECVASSOCEVNT))    ||
       ((notification.Content.sn_header.sn_type == SCTP_PEER_ADDR_CHANGE) && (notificationFlags & SCTP_RECVPADDREVNT))    ||
       ((notification.Content.sn_header.sn_type == SCTP_REMOTE_ERROR)     && (notificationFlags & SCTP_RECVPEERERR))      ||
@@ -1229,7 +1229,8 @@ void SCTPSocketMaster::addNotification(SCTPSocket*             socket,
 #ifdef PRINT_ASSOC_USECOUNT
       cout << association->UseCount << ". Notification Type = " << notification.Content.sn_header.sn_type << endl;
 #endif
-      if(socket->Flags & SCTPSocket::SSF_GlobalQueue) {
+      if( (socket->Flags & SCTPSocket::SSF_GlobalQueue) &&
+          (association->PeeledOff == false) ) {
          socket->GlobalQueue.addNotification(notification);
          socket->ReadReady = socket->hasData() || (socket->ConnectionRequests != NULL);
       }
